@@ -1,3 +1,4 @@
+#include "config.h"
 #include <glib.h>
 
 #include <sys/stat.h>
@@ -12,8 +13,8 @@ file_exists (const char *filename)
 }
 
 void
-pixbuf_init ()
+pixbuf_init (void)
 {
   if (file_exists ("../gdk-pixbuf/libpixbufloader-pnm.la"))
-    putenv ("GDK_PIXBUF_MODULEDIR=../gdk-pixbuf");
+    g_setenv ("GDK_PIXBUF_MODULE_FILE", "../gdk-pixbuf/loaders.cache", TRUE);
 }
