@@ -45,9 +45,14 @@
  */
 
 #include "config.h"
+
 #include "gtkbbox.h"
+
 #include "gtkorientable.h"
+#include "gtktypebuiltins.h"
 #include "gtkprivate.h"
+#include "gtksizerequest.h"
+
 #include "gtkintl.h"
 
 
@@ -76,8 +81,21 @@ static void gtk_button_box_get_property       (GObject           *object,
                                                guint              prop_id,
                                                GValue            *value,
                                                GParamSpec        *pspec);
-static void gtk_button_box_size_request       (GtkWidget         *widget,
-                                               GtkRequisition    *requisition);
+static void gtk_button_box_get_preferred_width            (GtkWidget *widget,
+                                                           gint      *minimum,
+                                                           gint      *natural);
+static void gtk_button_box_get_preferred_height           (GtkWidget *widget,
+                                                           gint      *minimum,
+                                                           gint      *natural);
+static void gtk_button_box_get_preferred_width_for_height (GtkWidget *widget,
+                                                           gint       height,
+                                                           gint      *minimum,
+                                                           gint      *natural);
+static void gtk_button_box_get_preferred_height_for_width (GtkWidget *widget,
+                                                           gint       width,
+                                                           gint      *minimum,
+                                                           gint      *natural);
+
 static void gtk_button_box_size_allocate      (GtkWidget         *widget,
                                                GtkAllocation     *allocation);
 static void gtk_button_box_remove             (GtkContainer      *container,
@@ -115,12 +133,16 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
   gobject_class->set_property = gtk_button_box_set_property;
   gobject_class->get_property = gtk_button_box_get_property;
 
-  widget_class->size_request = gtk_button_box_size_request;
+  widget_class->get_preferred_width = gtk_button_box_get_preferred_width;
+  widget_class->get_preferred_height = gtk_button_box_get_preferred_height;
+  widget_class->get_preferred_width_for_height = gtk_button_box_get_preferred_width_for_height;
+  widget_class->get_preferred_height_for_width = gtk_button_box_get_preferred_height_for_width;
   widget_class->size_allocate = gtk_button_box_size_allocate;
 
   container_class->remove = gtk_button_box_remove;
   container_class->set_child_property = gtk_button_box_set_child_property;
   container_class->get_child_property = gtk_button_box_get_child_property;
+  gtk_container_class_handle_border_width (container_class);
 
   /* FIXME we need to override the "spacing" property on GtkBox once
    * libgobject allows that.
@@ -396,7 +418,6 @@ gtk_button_box_child_requisition (GtkWidget  *widget,
                                   gint      **widths,
                                   gint      **heights)
 {
-  GtkButtonBoxPrivate *priv;
   GtkButtonBox *bbox;
   GList *children, *list;
   gint nchildren;
@@ -417,7 +438,6 @@ gtk_button_box_child_requisition (GtkWidget  *widget,
   g_return_if_fail (GTK_IS_BUTTON_BOX (widget));
 
   bbox = GTK_BUTTON_BOX (widget);
-  priv = bbox->priv;
 
   homogeneous = gtk_box_get_homogeneous (GTK_BOX (widget));
 
@@ -447,7 +467,8 @@ gtk_button_box_child_requisition (GtkWidget  *widget,
       if (gtk_widget_get_visible (child))
         {
           nchildren += 1;
-          gtk_widget_size_request (child, &child_requisition);
+          gtk_widget_get_preferred_size (child,
+                                         &child_requisition, NULL);
           avg_w += child_requisition.width + ipad_w;
           avg_h += child_requisition.height + ipad_h;
         }
@@ -474,7 +495,7 @@ gtk_button_box_child_requisition (GtkWidget  *widget,
           if (is_secondary)
             nsecondaries++;
 
-          gtk_widget_get_child_requisition (child, &child_requisition);
+          gtk_widget_get_preferred_size (child, &child_requisition, NULL);
 
           if (homogeneous || (child_requisition.width + ipad_w < avg_w * 1.5))
             {
@@ -529,7 +550,6 @@ gtk_button_box_size_request (GtkWidget      *widget,
   gint max_size;
   gint total_size;
   gint spacing;
-  guint border_width;
   GtkOrientation orientation;
   gint *widths;
   gint *heights;
@@ -600,10 +620,48 @@ gtk_button_box_size_request (GtkWidget      *widget,
       else
         requisition->width = max_size;
     }
+}
 
-  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-  requisition->width += border_width * 2;
-  requisition->height += border_width * 2;
+static void
+gtk_button_box_get_preferred_width (GtkWidget *widget,
+                                    gint      *minimum,
+                                    gint      *natural)
+{
+  GtkRequisition requisition;
+
+  gtk_button_box_size_request (widget, &requisition);
+
+  *minimum = *natural = requisition.width;
+}
+
+static void
+gtk_button_box_get_preferred_height (GtkWidget *widget,
+                                     gint      *minimum,
+                                     gint      *natural)
+{
+  GtkRequisition requisition;
+
+  gtk_button_box_size_request (widget, &requisition);
+
+  *minimum = *natural = requisition.height;
+}
+
+static void
+gtk_button_box_get_preferred_width_for_height (GtkWidget *widget,
+                                               gint       height,
+                                               gint      *minimum,
+                                               gint      *natural)
+{
+  gtk_button_box_get_preferred_width (widget, minimum, natural);
+}
+
+static void
+gtk_button_box_get_preferred_height_for_width (GtkWidget *widget,
+                                               gint       width,
+                                               gint      *minimum,
+                                               gint      *natural)
+{
+  gtk_button_box_get_preferred_height (widget, minimum, natural);
 }
 
 static void
@@ -625,7 +683,6 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
   gint height = 0;
   gint childspacing = 0;
   gint spacing;
-  guint border_width;
   GtkOrientation orientation;
   gint ipad_x, ipad_y;
   gint *widths;
@@ -639,7 +696,6 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
   bbox = GTK_BUTTON_BOX (widget);
   priv = bbox->priv;
 
-  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
   orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (widget));
   spacing = gtk_box_get_spacing (GTK_BOX (widget));
 
@@ -683,9 +739,9 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
   gtk_widget_set_allocation (widget, allocation);
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    width = allocation->width - border_width*2;
+    width = allocation->width;
   else
-    height = allocation->height - border_width*2;
+    height = allocation->height;
 
   switch (priv->layout_style)
     {
@@ -694,13 +750,13 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
         if (orientation == GTK_ORIENTATION_HORIZONTAL)
           {
             childspacing = (width - total_size) / (nvis_children + 1);
-            x = allocation->x + border_width + childspacing;
+            x = allocation->x + childspacing;
             secondary_x = x + primary_size + n_primaries * childspacing;
           }
         else
           {
             childspacing = (height - total_size) / (nvis_children + 1);
-            y = allocation->y + border_width + childspacing;
+            y = allocation->y + childspacing;
             secondary_y = y + primary_size + n_primaries * childspacing;
           }
 
@@ -713,31 +769,43 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
             if (nvis_children >= 2)
               {
                 childspacing = (width - total_size) / (nvis_children - 1);
-                x = allocation->x + border_width;
+                x = allocation->x;
                 secondary_x = x + primary_size + n_primaries * childspacing;
               }
-            else
+            else if (nvis_children == 1)
               {
-                /* one or zero children, just center */
+                /* one child, just center */
                 childspacing = width;
                 x = secondary_x = allocation->x
                                   + (allocation->width - widths[0]) / 2;
               }
+            else
+              {
+                /* zero children, meh */
+                childspacing = width;
+                x = secondary_x = allocation->x + allocation->width / 2;
+              }
           }
         else
           {
             if (nvis_children >= 2)
               {
                 childspacing = (height - total_size) / (nvis_children - 1);
-                y = allocation->y + border_width;
+                y = allocation->y;
                 secondary_y = y + primary_size + n_primaries * childspacing;
               }
+            else if (nvis_children == 1)
+              {
+                /* one child, just center */
+                childspacing = height;
+                y = secondary_y = allocation->y
+                                     + (allocation->height - heights[0]) / 2;
+              }
             else
               {
-                /* one or zero children, just center */
+                /* zero children, meh */
                 childspacing = height;
-                y = secondary_y = allocation->y
-                        + (allocation->height - heights[0]) / 2;
+                y = secondary_y = allocation->y + allocation->height / 2;
               }
           }
 
@@ -748,16 +816,16 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
         if (orientation == GTK_ORIENTATION_HORIZONTAL)
           {
             childspacing = spacing;
-            x = allocation->x + border_width;
+            x = allocation->x;
             secondary_x = allocation->x + allocation->width
-              - secondary_size - spacing * (n_secondaries - 1) - border_width;
+              - secondary_size - spacing * (n_secondaries - 1);
           }
         else
           {
             childspacing = spacing;
-            y = allocation->y + border_width;
+            y = allocation->y;
             secondary_y = allocation->y + allocation->height
-              - secondary_size - spacing * (n_secondaries - 1) - border_width;
+              - secondary_size - spacing * (n_secondaries - 1);
           }
 
         break;
@@ -768,15 +836,15 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
           {
             childspacing = spacing;
             x = allocation->x + allocation->width
-              - primary_size - spacing * (n_primaries - 1) - border_width;
-            secondary_x = allocation->x + border_width;
+              - primary_size - spacing * (n_primaries - 1);
+            secondary_x = allocation->x;
           }
         else
           {
             childspacing = spacing;
             y = allocation->y + allocation->height
-              - primary_size - spacing * (n_primaries - 1) - border_width;
-            secondary_y = allocation->y + border_width;
+              - primary_size - spacing * (n_primaries - 1);
+            secondary_y = allocation->y;
           }
 
         break;
@@ -790,7 +858,7 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
               (allocation->width
                - (primary_size + spacing * (n_primaries - 1))) / 2
               + (secondary_size + n_secondaries * spacing) / 2;
-            secondary_x = allocation->x + border_width;
+            secondary_x = allocation->x;
           }
         else
           {
@@ -799,7 +867,7 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
               (allocation->height
                - (primary_size + spacing * (n_primaries - 1))) / 2
               + (secondary_size + n_secondaries * spacing) / 2;
-            secondary_y = allocation->y + border_width;
+            secondary_y = allocation->y;
           }
 
         break;
