@@ -62,8 +62,6 @@ struct _GtkContainerClass
 {
   GtkWidgetClass parent_class;
 
-  unsigned int handle_border_width : 1;
-
   void    (*add)       		(GtkContainer	 *container,
 				 GtkWidget	 *widget);
   void    (*remove)    		(GtkContainer	 *container,
@@ -74,7 +72,7 @@ struct _GtkContainerClass
 				 GtkCallback	  callback,
 				 gpointer	  callback_data);
   void    (*set_focus_child)	(GtkContainer	 *container,
-				 GtkWidget	 *widget);
+				 GtkWidget	 *child);
   GType   (*child_type)		(GtkContainer	 *container);
   gchar*  (*composite_name)	(GtkContainer	 *container,
 				 GtkWidget	 *child);
@@ -88,12 +86,23 @@ struct _GtkContainerClass
 				 guint            property_id,
 				 GValue          *value,
 				 GParamSpec      *pspec);
+  GtkWidgetPath * (*get_path_for_child) (GtkContainer *container,
+                                         GtkWidget    *child);
+
+
+  /*< private >*/
+
+  unsigned int _handle_border_width : 1;
 
   /* Padding for future expansion */
   void (*_gtk_reserved1) (void);
   void (*_gtk_reserved2) (void);
   void (*_gtk_reserved3) (void);
   void (*_gtk_reserved4) (void);
+  void (*_gtk_reserved5) (void);
+  void (*_gtk_reserved6) (void);
+  void (*_gtk_reserved7) (void);
+  void (*_gtk_reserved8) (void);
 };
 
 
@@ -224,6 +233,9 @@ void     _gtk_container_set_need_resize      (GtkContainer     *container,
                                               gboolean          need_resize);
 gboolean _gtk_container_get_reallocate_redraws (GtkContainer   *container);
 
+GtkWidgetPath * gtk_container_get_path_for_child (GtkContainer      *container,
+                                                  GtkWidget         *child);
+
 G_END_DECLS
 
 #endif /* __GTK_CONTAINER_H__ */
