@@ -35,16 +35,12 @@ typedef enum
   GTK_RBNODE_IS_PARENT = 1 << 2,
   GTK_RBNODE_IS_SELECTED = 1 << 3,
   GTK_RBNODE_IS_PRELIT = 1 << 4,
-  GTK_RBNODE_IS_SEMI_COLLAPSED = 1 << 5,
-  GTK_RBNODE_IS_SEMI_EXPANDED = 1 << 6,
   GTK_RBNODE_INVALID = 1 << 7,
   GTK_RBNODE_COLUMN_INVALID = 1 << 8,
   GTK_RBNODE_DESCENDANTS_INVALID = 1 << 9,
   GTK_RBNODE_NON_COLORS = GTK_RBNODE_IS_PARENT |
   			  GTK_RBNODE_IS_SELECTED |
   			  GTK_RBNODE_IS_PRELIT |
-                          GTK_RBNODE_IS_SEMI_COLLAPSED |
-                          GTK_RBNODE_IS_SEMI_EXPANDED |
                           GTK_RBNODE_INVALID |
                           GTK_RBNODE_COLUMN_INVALID |
                           GTK_RBNODE_DESCENDANTS_INVALID
@@ -111,8 +107,6 @@ struct _GtkRBNode
 #define GTK_RBNODE_FLAG_SET(node, flag) 	(node?(((node->flags&flag)==flag)?TRUE:FALSE):FALSE)
 
 
-void       _gtk_rbtree_push_allocator   (GAllocator             *allocator);
-void       _gtk_rbtree_pop_allocator    (void);
 GtkRBTree *_gtk_rbtree_new              (void);
 void       _gtk_rbtree_free             (GtkRBTree              *tree);
 void       _gtk_rbtree_remove           (GtkRBTree              *tree);
