@@ -26,7 +26,7 @@
 
 #include "config.h"
 
-#include "gtkorientable.h"
+#include "gtkorientableprivate.h"
 #include "gtkseparator.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
@@ -61,11 +61,16 @@ static void       gtk_separator_get_property (GObject        *object,
                                               guint           prop_id,
                                               GValue         *value,
                                               GParamSpec     *pspec);
-
-static void       gtk_separator_size_request (GtkWidget      *widget,
-                                              GtkRequisition *requisition);
-static gboolean   gtk_separator_expose       (GtkWidget      *widget,
-                                              GdkEventExpose *event);
+static void       gtk_separator_get_preferred_width
+                                             (GtkWidget      *widget,
+                                              gint           *minimum,
+                                              gint           *natural);
+static void       gtk_separator_get_preferred_height
+                                             (GtkWidget      *widget,
+                                              gint           *minimum,
+                                              gint           *natural);
+static gboolean   gtk_separator_draw         (GtkWidget      *widget,
+                                              cairo_t        *cr);
 
 
 G_DEFINE_TYPE_WITH_CODE (GtkSeparator, gtk_separator, GTK_TYPE_WIDGET,
@@ -82,12 +87,12 @@ gtk_separator_class_init (GtkSeparatorClass *class)
   object_class->set_property = gtk_separator_set_property;
   object_class->get_property = gtk_separator_get_property;
 
-  widget_class->size_request = gtk_separator_size_request;
-  widget_class->expose_event = gtk_separator_expose;
+  widget_class->get_preferred_width = gtk_separator_get_preferred_width;
+  widget_class->get_preferred_height = gtk_separator_get_preferred_height;
 
-  g_object_class_override_property (object_class,
-                                    PROP_ORIENTATION,
-                                    "orientation");
+  widget_class->draw         = gtk_separator_draw;
+
+  g_object_class_override_property (object_class, PROP_ORIENTATION, "orientation");
 
   g_type_class_add_private (object_class, sizeof (GtkSeparatorPrivate));
 }
@@ -96,6 +101,7 @@ static void
 gtk_separator_init (GtkSeparator *separator)
 {
   GtkSeparatorPrivate *private;
+  GtkStyleContext *context;
 
   separator->priv = G_TYPE_INSTANCE_GET_PRIVATE (separator,
                                                  GTK_TYPE_SEPARATOR,
@@ -105,6 +111,9 @@ gtk_separator_init (GtkSeparator *separator)
   gtk_widget_set_has_window (GTK_WIDGET (separator), FALSE);
 
   private->orientation = GTK_ORIENTATION_HORIZONTAL;
+
+  context = gtk_widget_get_style_context (GTK_WIDGET (separator));
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_SEPARATOR);
 }
 
 static void
@@ -120,6 +129,7 @@ gtk_separator_set_property (GObject      *object,
     {
     case PROP_ORIENTATION:
       private->orientation = g_value_get_enum (value);
+      _gtk_orientable_set_style_classes (GTK_ORIENTABLE (object));
       gtk_widget_queue_resize (GTK_WIDGET (object));
       break;
     default:
@@ -149,107 +159,115 @@ gtk_separator_get_property (GObject    *object,
 }
 
 static void
-gtk_separator_size_request (GtkWidget      *widget,
-                            GtkRequisition *requisition)
+gtk_separator_get_preferred_size (GtkWidget      *widget,
+                                  GtkOrientation  orientation,
+                                  gint           *minimum,
+                                  gint           *natural)
 {
   GtkSeparator *separator = GTK_SEPARATOR (widget);
   GtkSeparatorPrivate *private = separator->priv;
-  GtkStyle *style;
-  gboolean wide_separators;
-  gint     separator_width;
-  gint     separator_height;
+  GtkStyleContext *context;
+  GtkStateFlags state;
+  GtkBorder border;
+  gboolean wide_sep;
+  gint     sep_width;
+  gint     sep_height;
+
+  context = gtk_widget_get_style_context (widget);
+  state = gtk_widget_get_state_flags (widget);
+  gtk_style_context_get_border (context, state, &border);
 
-  style = gtk_widget_get_style (widget);
   gtk_widget_style_get (widget,
-                        "wide-separators",  &wide_separators,
-                        "separator-width",  &separator_width,
-                        "separator-height", &separator_height,
+                        "wide-separators",  &wide_sep,
+                        "separator-width",  &sep_width,
+                        "separator-height", &sep_height,
                         NULL);
 
-  requisition->width  = 1;
-  requisition->height = 1;
-
-  if (private->orientation == GTK_ORIENTATION_HORIZONTAL)
+  if (orientation == private->orientation)
     {
-      if (wide_separators)
-        requisition->height = separator_height;
-      else
-        requisition->height = style->ythickness;
+      *minimum = *natural = 1;
+    }
+  else if (orientation == GTK_ORIENTATION_VERTICAL)
+    {
+      *minimum = *natural = wide_sep ? sep_height : border.top;
     }
   else
     {
-      if (wide_separators)
-        requisition->width = separator_width;
-      else
-        requisition->width = style->xthickness;
+      *minimum = *natural = wide_sep ? sep_width : border.left;
     }
 }
 
+static void
+gtk_separator_get_preferred_width (GtkWidget *widget,
+                                   gint      *minimum,
+                                   gint      *natural)
+{
+  gtk_separator_get_preferred_size (widget, GTK_ORIENTATION_HORIZONTAL, minimum, natural);
+}
+
+static void
+gtk_separator_get_preferred_height (GtkWidget *widget,
+                                    gint      *minimum,
+                                    gint      *natural)
+{
+  gtk_separator_get_preferred_size (widget, GTK_ORIENTATION_VERTICAL, minimum, natural);
+}
+
 static gboolean
-gtk_separator_expose (GtkWidget      *widget,
-                      GdkEventExpose *event)
+gtk_separator_draw (GtkWidget *widget,
+                    cairo_t   *cr)
 {
   GtkSeparator *separator = GTK_SEPARATOR (widget);
   GtkSeparatorPrivate *private = separator->priv;
-  GtkAllocation allocation;
-  GtkStateType state;
-  GtkStyle *style;
-  GdkWindow *window;
+  GtkStateFlags state;
+  GtkStyleContext *context;
+  GtkBorder padding;
   gboolean wide_separators;
-  gint     separator_width;
-  gint     separator_height;
+  gint separator_width;
+  gint separator_height;
+  int width, height;
 
-  if (!gtk_widget_is_drawable (widget))
-    return FALSE;
-
-  style = gtk_widget_get_style (widget);
+  context = gtk_widget_get_style_context (widget);
   gtk_widget_style_get (widget,
                         "wide-separators",  &wide_separators,
                         "separator-width",  &separator_width,
                         "separator-height", &separator_height,
                         NULL);
 
-  window = gtk_widget_get_window (widget);
-  state = gtk_widget_get_state (widget);
-  gtk_widget_get_allocation (widget, &allocation);
+  state = gtk_widget_get_state_flags (widget);
+  width = gtk_widget_get_allocated_width (widget);
+  height = gtk_widget_get_allocated_height (widget);
+
+  gtk_style_context_get_padding (context, state, &padding);
+
+  gtk_style_context_save (context);
+  gtk_style_context_set_state (context, state);
 
   if (private->orientation == GTK_ORIENTATION_HORIZONTAL)
     {
       if (wide_separators)
-        gtk_paint_box (style, window,
-                       state, GTK_SHADOW_ETCHED_OUT,
-                       &event->area, widget, "hseparator",
-                       allocation.x,
-                       allocation.y + (allocation.height - separator_height) / 2,
-                       allocation.width,
-                       separator_height);
+        gtk_render_frame (context, cr,
+                          0, (height - separator_height) / 2,
+                          width, separator_height);
       else
-        gtk_paint_hline (style, window,
-                         state,
-                         &event->area, widget, "hseparator",
-                         allocation.x,
-                         allocation.x + allocation.width - 1,
-                         allocation.y + (allocation.height - style->ythickness) / 2);
+        gtk_render_line (context, cr,
+                         0, (height - padding.top) / 2,
+                         width - 1, (height - padding.top) / 2);
     }
   else
     {
       if (wide_separators)
-        gtk_paint_box (style, window,
-                       state, GTK_SHADOW_ETCHED_OUT,
-                       &event->area, widget, "vseparator",
-                       allocation.x + (allocation.width - separator_width) / 2,
-                       allocation.y,
-                       separator_width,
-                       allocation.height);
+        gtk_render_frame (context, cr,
+                          (width - separator_width) / 2, 0,
+                          separator_width, height);
       else
-        gtk_paint_vline (style, window,
-                         state,
-                         &event->area, widget, "vseparator",
-                         allocation.y,
-                         allocation.y + allocation.height - 1,
-                         allocation.x + (allocation.width - style->xthickness) / 2);
+        gtk_render_line (context, cr,
+                         (width - padding.left) / 2, 0,
+                         (width - padding.left) / 2, height - 1);
     }
 
+  gtk_style_context_restore (context);
+
   return FALSE;
 }
 
@@ -262,7 +280,7 @@ gtk_separator_expose (GtkWidget      *widget,
  * Return value: a new #GtkSeparator.
  *
  * Since: 3.0
- **/
+ */
 GtkWidget *
 gtk_separator_new (GtkOrientation orientation)
 {
