@@ -20,13 +20,21 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if !defined (__GTKX_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtkx.h> can be included directly."
+#endif
+
 #ifndef __GTK_SOCKET_H__
 #define __GTK_SOCKET_H__
 
-#include <gtk/gtkcontainer.h>
+#include <gtk/gtk.h>
+
+#ifdef GDK_WINDOWING_X11
+
+#include <gdk/gdkx.h>
 
 G_BEGIN_DECLS
 
@@ -40,29 +48,13 @@ G_BEGIN_DECLS
 
 typedef struct _GtkSocket        GtkSocket;
 typedef struct _GtkSocketClass   GtkSocketClass;
+typedef struct _GtkSocketPrivate GtkSocketPrivate;
 
 struct _GtkSocket
 {
   GtkContainer container;
 
-  guint16 request_width;
-  guint16 request_height;
-  guint16 current_width;
-  guint16 current_height;
-  
-  GdkWindow *plug_window;
-  GtkWidget *plug_widget;
-
-  gshort xembed_version;	/* -1 == not xembed */
-  guint same_app : 1;
-  guint focus_in : 1;
-  guint have_size : 1;
-  guint need_map : 1;
-  guint is_mapped : 1;
-  guint active : 1;
-  
-  GtkAccelGroup *accel_group;
-  GtkWidget *toplevel;
+  GtkSocketPrivate *priv;
 };
 
 struct _GtkSocketClass
@@ -79,19 +71,15 @@ struct _GtkSocketClass
   void (*_gtk_reserved4) (void);
 };
 
-
-GType          gtk_socket_get_type (void) G_GNUC_CONST;
-GtkWidget*     gtk_socket_new      (void);
-
-void            gtk_socket_add_id (GtkSocket       *socket_,
-				   GdkNativeWindow  window_id);
-GdkNativeWindow gtk_socket_get_id (GtkSocket       *socket_);
-
-#ifndef GTK_DISABLE_DEPRECATED
-void           gtk_socket_steal    (GtkSocket      *socket_,
-				    GdkNativeWindow wid);
-#endif /* GTK_DISABLE_DEPRECATED */
+GType      gtk_socket_get_type        (void) G_GNUC_CONST;
+GtkWidget *gtk_socket_new             (void);
+void       gtk_socket_add_id          (GtkSocket *socket_,
+                                       Window     window);
+Window     gtk_socket_get_id          (GtkSocket *socket_);
+GdkWindow *gtk_socket_get_plug_window (GtkSocket *socket_);
 
 G_END_DECLS
 
+#endif /* GDK_WINDOWING_X11 */
+
 #endif /* __GTK_SOCKET_H__ */
