@@ -119,43 +119,6 @@ gtk_spinner_class_init (GtkSpinnerClass *klass)
                                                          P_("Whether the spinner is active"),
                                                          FALSE,
                                                          G_PARAM_READWRITE));
-  /**
-   * GtkSpinner:num-steps:
-   *
-   * The number of steps for the spinner to complete a full loop.
-   * The animation will complete a full cycle in one second by default
-   * (see the #GtkSpinner:cycle-duration style property).
-   *
-   * Since: 2.20
-   *
-   * Deprecated: 3.0
-   */
-  gtk_widget_class_install_style_property (widget_class,
-                                           g_param_spec_uint ("num-steps",
-                                                             P_("Number of steps"),
-                                                             P_("The number of steps for the spinner to complete a full loop. The animation will complete a full cycle in one second by default (see #GtkSpinner:cycle-duration)."),
-                                                             1,
-                                                             G_MAXUINT,
-                                                             12,
-                                                             G_PARAM_READABLE));
-
-  /**
-   * GtkSpinner:cycle-duration:
-   *
-   * The duration in milliseconds for the spinner to complete a full cycle.
-   *
-   * Since: 2.20
-   *
-   * Deprecated: 3.0
-   */
-  gtk_widget_class_install_style_property (widget_class,
-                                           g_param_spec_uint ("cycle-duration",
-                                                             P_("Animation duration"),
-                                                             P_("The length of time in milliseconds for the spinner to complete a full loop"),
-                                                             500,
-                                                             G_MAXUINT,
-                                                             1000,
-                                                             G_PARAM_READABLE));
 }
 
 static void
@@ -239,11 +202,9 @@ static gboolean
 gtk_spinner_draw (GtkWidget *widget,
                   cairo_t   *cr)
 {
-  GtkSpinnerPrivate *priv;
   GtkStyleContext *context;
   GtkStateFlags state;
 
-  priv = GTK_SPINNER (widget)->priv;
   context = gtk_widget_get_style_context (widget);
   state = gtk_widget_get_state_flags (widget);
 
