@@ -827,22 +827,6 @@ gtk_toolbar_style_get_type (void)
     return etype;
 }
 
-GType
-gtk_update_type_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_UPDATE_CONTINUOUS, "GTK_UPDATE_CONTINUOUS", "continuous" },
-            { GTK_UPDATE_DISCONTINUOUS, "GTK_UPDATE_DISCONTINUOUS", "discontinuous" },
-            { GTK_UPDATE_DELAYED, "GTK_UPDATE_DELAYED", "delayed" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkUpdateType"), values);
-    }
-    return etype;
-}
-
 GType
 gtk_window_position_get_type (void)
 {
@@ -1150,6 +1134,7 @@ gtk_state_flags_get_type (void)
     static GType etype = 0;
     if (G_UNLIKELY(etype == 0)) {
         static const GFlagsValue values[] = {
+            { GTK_STATE_FLAG_NORMAL, "GTK_STATE_FLAG_NORMAL", "normal" },
             { GTK_STATE_FLAG_ACTIVE, "GTK_STATE_FLAG_ACTIVE", "active" },
             { GTK_STATE_FLAG_PRELIGHT, "GTK_STATE_FLAG_PRELIGHT", "prelight" },
             { GTK_STATE_FLAG_SELECTED, "GTK_STATE_FLAG_SELECTED", "selected" },
